package pps.core.task.job;

import cn.hutool.core.util.ObjectUtil;
import pps.cloud.prediction.service.IThirdDataAccessCloudService;
import pps.core.common.cache.TaskLockCache;
import pps.core.task.constant.TaskConstant;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.data.XServiceResult;
import xstartup.service.job.XJob;
import xstartup.service.job.annotation.XCronTrigger;

/**
 * 能耗分析定时任务
 *
 * @author ZWT
 * @date 2024/03/20 14:17
 */
@XText("能耗分析定时任务")
@XService
public class EnergyConsumptionAnalysisJob implements XJob {

    /**
     * 每天0点30执行
     *
     * @param xContext x上下文
     * @return {@link XServiceResult}
     */
    @XCronTrigger(value = TaskConstant.EVERY_DAY_30)
    @Override
    public XServiceResult execute(XContext xContext) {
        xContext.getLogger().info("------ EnergyConsumptionAnalysisJob start:{}", System.currentTimeMillis());
        String key = TaskConstant.TASK_LOCK_KEY + "EnergyConsumptionAnalysisJob";
        TaskLockCache exist = TaskLockCache.exist(xContext, key);
        if (ObjectUtil.isNull(exist)) {
            TaskLockCache cache = new TaskLockCache();
            cache.setRedisKey(key);
            cache.setRedisValue(key);
            TaskLockCache.set(xContext, cache);
            try {
                IThirdDataAccessCloudService service = xContext.getBean(IThirdDataAccessCloudService.class);
                XServiceResult result = service.energyConsumptionAnalysisJob(xContext);
                result.throwIfFail();
            } catch (Exception e) {
                xContext.getLogger().error("------ EnergyConsumptionAnalysisJob Exception: ", e);
            } finally {
                xContext.getLogger().info("------ EnergyConsumptionAnalysisJob end:{}", System.currentTimeMillis());
                TaskLockCache.delete(xContext, key);
            }
        }
        return XServiceResult.OK;
    }
}
