package pps.core.prediction.utils;

import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.text.CharSequenceUtil;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import pps.core.auth.HttpRequestClient;
import pps.core.common.cache.ThirdPartyConfigCache;
import pps.core.prediction.constant.ThirdPartyApiConstant;
import xstartup.base.XContext;
import xstartup.base.XStartup;
import xstartup.base.exception.XServiceException;
import xstartup.base.util.XHttpUtils;

import java.util.HashMap;
import java.util.Map;

/**
 * 常用服务调用工具类
 *
 * @author ZWT
 * @date 2024/03/18 15:07
 */
public class ServiceUtil {

    /**
     * 调用接口(长庆)
     *
     * @param context 上下文
     * @param api     应用程序编程接口
     * @param param   参数
     * @return {@link String}
     */
    public static String doPostFormCq(XContext context, String api, Map<String, Object> param) {
        String url = XStartup.getCurrent().getProperty("third-party.chang-qing.url");
        String token = ServiceUtil.getToken(context);
        HttpRequestClient client = new HttpRequestClient(token);
        String resultStr = client.doPostForm(url + api, param);
        JSONObject jsonObject = JSON.parseObject(resultStr);
        context.getLogger().info("**** doPostFormCq api: {}", api);
        context.getLogger().info("**** doPostFormCq resultStr: {}", resultStr);
        return jsonObject.getString("data");
    }

    /**
     * 获取长庆TOKEN
     *
     * @param context 上下文
     * @return {@link String}
     */
    public static String getToken(XContext context) {
        String token;
        ThirdPartyConfigCache exist = ThirdPartyConfigCache.exist(context, ThirdPartyApiConstant.CQ_TOKEN_CACHE_KEY);
        if (ObjectUtil.isNull(exist)) {
            token = ServiceUtil.getCqToken(context);
        } else {
            long duration = (long) exist.getDuration();
            if (duration - DateUtil.between(exist.getCurrentDate(), DateUtil.date(), DateUnit.SECOND) < ThirdPartyApiConstant.MAX_SECOND) {
                token = ServiceUtil.getCqToken(context);
            } else {
                token = exist.getCode();
            }
        }
        return token;
    }

    /*----------------------------------- 私有方法 -----------------------------------*/

    /**
     * 获取长庆Token
     *
     * @return {@link String}
     */
    private static String getCqToken(XContext context) {
        String url = XStartup.getCurrent().getProperty("third-party.chang-qing.url");
        url += ThirdPartyApiConstant.CQ_TOKEN;
        Map<String, String> param = new HashMap<>(4);
        param.put("UserAccount", "PvGroup");
        param.put("UserPassWord", "PvGroup@147369");
        String result = XHttpUtils.postAsForm(url, param);
        JSONObject jsonObject = JSON.parseObject(result);
        String token = jsonObject.getString("token");
        if (CharSequenceUtil.isBlank(token)) {
            throw new XServiceException("获取长庆Token失败");
        }
        ServiceUtil.setThirdPartyConfigCache(context, token);
        return token;
    }

    /**
     * 设置缓存
     *
     * @param context 上下文
     * @param token   代币
     */
    private static void setThirdPartyConfigCache(XContext context, String token) {
        ThirdPartyConfigCache cache = new ThirdPartyConfigCache();
        cache.setCodeKey(ThirdPartyApiConstant.CQ_TOKEN_CACHE_KEY);
        cache.setCode(token);
        cache.setValidity(90);
        cache.setCurrentDate(DateUtil.date());
        ThirdPartyConfigCache.set(context, cache);
    }
}
